@@ -1,6 +1,6 @@
 #!/bin/bash
 
-python -m pip install --no-deps --ignore-installed .
+$PYTHON -m pip install . --no-deps --ignore-installed --no-cache-dir -vvv
 
 # Copy the [de]activate scripts to $PREFIX/etc/conda/[de]activate.d.
 # This will allow them to be run on environment activation.
