@@ -1,8 +1,12 @@
 # -*- coding: utf-8 -*-
+"""Reuters topic classification dataset.
+"""
 from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+
 from ..utils.data_utils import get_file
 from ..preprocessing.sequence import _remove_long_seq
-from six.moves import zip
 import numpy as np
 import json
 import warnings
@@ -46,14 +50,17 @@ def load_data(path='reuters.npz', num_words=None, skip_top=0,
     if kwargs:
         raise TypeError('Unrecognized keyword arguments: ' + str(kwargs))
 
-    path = get_file(path, origin='https://s3.amazonaws.com/text-datasets/reuters.npz')
+    path = get_file(path,
+                    origin='https://s3.amazonaws.com/text-datasets/reuters.npz',
+                    file_hash='87aedbeb0cb229e378797a632c1997b6')
     with np.load(path) as f:
         xs, labels = f['x'], f['y']
 
     np.random.seed(seed)
-    np.random.shuffle(xs)
-    np.random.seed(seed)
-    np.random.shuffle(labels)
+    indices = np.arange(len(xs))
+    np.random.shuffle(indices)
+    xs = xs[indices]
+    labels = labels[indices]
 
     if start_char is not None:
         xs = [[start_char] + [w + index_from for w in x] for x in xs]
@@ -70,9 +77,9 @@ def load_data(path='reuters.npz', num_words=None, skip_top=0,
     # reserve 'index_from' (=3 by default) characters:
     # 0 (padding), 1 (start), 2 (OOV)
     if oov_char is not None:
-        xs = [[w if (skip_top <= w < num_words) else oov_char for w in x] for x in xs]
+        xs = [[w if skip_top <= w < num_words else oov_char for w in x] for x in xs]
     else:
-        xs = [[w for w in x if (skip_top <= w < num_words)] for x in xs]
+        xs = [[w for w in x if skip_top <= w < num_words] for x in xs]
 
     idx = int(len(xs) * (1 - test_split))
     x_train, y_train = np.array(xs[:idx]), np.array(labels[:idx])
@@ -82,7 +89,7 @@ def load_data(path='reuters.npz', num_words=None, skip_top=0,
 
 
 def get_word_index(path='reuters_word_index.json'):
-    """Retrieves the dictionary mapping word indices back to words.
+    """Retrieves the dictionary mapping words to word indices.
 
     # Arguments
         path: where to cache the data (relative to `~/.keras/dataset`).
@@ -90,7 +97,9 @@ def get_word_index(path='reuters_word_index.json'):
     # Returns
         The word index dictionary.
     """
-    path = get_file(path, origin='https://s3.amazonaws.com/text-datasets/reuters_word_index.json')
+    path = get_file(path,
+                    origin='https://s3.amazonaws.com/text-datasets/reuters_word_index.json',
+                    file_hash='4d44cc38712099c9e383dc6e5f11a921')
     f = open(path)
     data = json.load(f)
     f.close()
