@@ -15,7 +15,7 @@ from .recurrent import _standardize_args
 
 import numpy as np
 import warnings
-from ..engine import InputSpec, Layer
+from ..engine.base_layer import InputSpec, Layer
 from ..utils import conv_utils
 from ..legacy import interfaces
 from ..legacy.layers import Recurrent, ConvRecurrent2D
@@ -563,7 +563,7 @@ class ConvLSTM2DCell(Layer):
         self.kernel_size = conv_utils.normalize_tuple(kernel_size, 2, 'kernel_size')
         self.strides = conv_utils.normalize_tuple(strides, 2, 'strides')
         self.padding = conv_utils.normalize_padding(padding)
-        self.data_format = conv_utils.normalize_data_format(data_format)
+        self.data_format = K.normalize_data_format(data_format)
         self.dilation_rate = conv_utils.normalize_tuple(dilation_rate, 2, 'dilation_rate')
         self.activation = activations.get(activation)
         self.recurrent_activation = activations.get(recurrent_activation)
@@ -622,15 +622,20 @@ class ConvLSTM2DCell(Layer):
             regularizer=self.recurrent_regularizer,
             constraint=self.recurrent_constraint)
         if self.use_bias:
+            if self.unit_forget_bias:
+                def bias_initializer(_, *args, **kwargs):
+                    return K.concatenate([
+                        self.bias_initializer((self.filters,), *args, **kwargs),
+                        initializers.Ones()((self.filters,), *args, **kwargs),
+                        self.bias_initializer((self.filters * 2,), *args, **kwargs),
+                    ])
+            else:
+                bias_initializer = self.bias_initializer
             self.bias = self.add_weight(shape=(self.filters * 4,),
-                                        initializer=self.bias_initializer,
                                         name='bias',
+                                        initializer=bias_initializer,
                                         regularizer=self.bias_regularizer,
                                         constraint=self.bias_constraint)
-            if self.unit_forget_bias:
-                bias_value = np.zeros((self.filters * 4,))
-                bias_value[self.filters: self.filters * 2] = 1.
-                K.set_value(self.bias, bias_value)
         else:
             self.bias = None
 
@@ -863,7 +868,7 @@ class ConvLSTM2D(ConvRNN2D):
             5D tensor with shape:
             `(samples, time, rows, cols, channels)`
 
-     # Output shape
+    # Output shape
         - if `return_sequences`
              - if data_format='channels_first'
                 5D tensor with shape:
