@@ -1,473 +1,20 @@
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+
 import numpy as np
-import types as python_types
 import warnings
 
-from ..engine.topology import Layer, InputSpec
+from ..engine import Layer, InputSpec
 from .. import backend as K
-from ..utils.generic_utils import func_dump, func_load, has_arg
+from ..utils import conv_utils
+from ..utils.generic_utils import to_list
 from .. import regularizers
 from .. import constraints
 from .. import activations
 from .. import initializers
 
 
-class Merge(Layer):
-    """A `Merge` layer can be used to merge a list of tensors
-    into a single tensor, following some merge `mode`.
-    # Example
-    ```python
-    model1 = Sequential()
-    model1.add(Dense(32, input_dim=32))
-    model2 = Sequential()
-    model2.add(Dense(32, input_dim=32))
-    merged_model = Sequential()
-    merged_model.add(Merge([model1, model2], mode='concat', concat_axis=1))
-    ```
-    # Arguments
-        layers: Can be a list of Keras tensors or
-            a list of layer instances. Must be more
-            than one layer/tensor.
-        mode: String or lambda/function. If string, must be one
-            of: 'sum', 'mul', 'concat', 'ave', 'cos', 'dot', 'max'.
-            If lambda/function, it should take as input a list of tensors
-            and return a single tensor.
-        concat_axis: Integer, axis to use in mode `concat`.
-        dot_axes: Integer or tuple of integers,
-            axes to use in mode `dot` or `cos`.
-        output_shape: Either a shape tuple (tuple of integers),
-            or a lambda/function
-            to compute `output_shape`
-            (only if merge mode is a lambda/function).
-            If the argument is a tuple,
-            it should be expected output shape, *not* including the batch size
-            (same convention as the `input_shape` argument in layers).
-            If the argument is callable,
-            it should take as input a list of shape tuples
-            (1:1 mapping to input tensors)
-            and return a single shape tuple, including the
-            batch size (same convention as the
-            `compute_output_shape` method of layers).
-        node_indices: Optional list of integers containing
-            the output node index for each input layer
-            (in case some input layers have multiple output nodes).
-            will default to an array of 0s if not provided.
-        tensor_indices: Optional list of indices of output tensors
-            to consider for merging
-            (in case some input layer node returns multiple tensors).
-        output_mask: Mask or lambda/function to compute the output mask (only
-            if merge mode is a lambda/function). If the latter case, it should
-            take as input a list of masks and return a single mask.
-    """
-
-    def __init__(self, layers=None, mode='sum', concat_axis=-1,
-                 dot_axes=-1, output_shape=None, output_mask=None,
-                 arguments=None, node_indices=None, tensor_indices=None,
-                 name=None):
-        warnings.warn('The `Merge` layer is deprecated '
-                      'and will be removed after 08/2017. '
-                      'Use instead layers from `keras.layers.merge`, '
-                      'e.g. `add`, `concatenate`, etc.', stacklevel=2)
-        self.layers = layers
-        self.mode = mode
-        self.concat_axis = concat_axis
-        self.dot_axes = dot_axes
-        self._output_shape = output_shape
-        self.node_indices = node_indices
-        self._output_mask = output_mask
-        self.arguments = arguments if arguments else {}
-        self._initial_weights = None
-        self._updates = []
-        self._losses = []
-        self._per_input_updates = {}
-        self._per_input_losses = {}
-
-        # Layer parameters.
-        self.inbound_nodes = []
-        self.outbound_nodes = []
-        self.constraints = {}
-        self._trainable_weights = []
-        self._non_trainable_weights = []
-        self.supports_masking = True
-        self.uses_learning_phase = False
-        self.input_spec = None  # Compatible with anything.
-        if not name:
-            prefix = self.__class__.__name__.lower()
-            name = prefix + '_' + str(K.get_uid(prefix))
-        self.name = name
-
-        if layers:
-            # This exists for backwards compatibility.
-            # equivalent to:
-            # merge = Merge(layers=None)
-            # output = merge([input_tensor_1, input_tensor_2])
-            if not node_indices:
-                # By default we connect to
-                # the 1st output stream in the input layer.
-                node_indices = [0 for _ in range(len(layers))]
-            if not tensor_indices:
-                tensor_indices = [0 for _ in range(len(layers))]
-            self._arguments_validation(layers, mode,
-                                       concat_axis, dot_axes,
-                                       node_indices, tensor_indices)
-            self.built = True
-            input_tensors = []
-            input_masks = []
-            for i, layer in enumerate(layers):
-                node_index = node_indices[i]
-                tensor_index = tensor_indices[i]
-                inbound_node = layer.inbound_nodes[node_index]
-                input_tensors.append(inbound_node.output_tensors[tensor_index])
-                input_masks.append(inbound_node.output_masks[tensor_index])
-            self(input_tensors, mask=input_masks)
-        else:
-            self.built = False
-
-    def _arguments_validation(self, layers, mode, concat_axis, dot_axes,
-                              node_indices, tensor_indices):
-        """Validates user-passed arguments and raises exceptions
-        as appropriate.
-        """
-        if not callable(mode):
-            if mode not in {'sum', 'mul', 'concat', 'ave', 'cos', 'dot', 'max'}:
-                raise ValueError('Invalid merge mode: ' + str(mode))
-        if not isinstance(layers, (list, tuple)) or len(layers) < 2:
-            raise TypeError('A Merge should only be applied to a list of '
-                            'layers with at least 2 elements. Found: ' +
-                            str(layers))
-
-        if tensor_indices is None:
-            tensor_indices = [None for _ in range(len(layers))]
-
-        input_shapes = []
-        for i, layer in enumerate(layers):
-            layer_output_shape = layer.get_output_shape_at(node_indices[i])
-            if isinstance(layer_output_shape, list):
-                # Case: the layer has multiple output tensors
-                # and we only need a specific one.
-                layer_output_shape = layer_output_shape[tensor_indices[i]]
-            input_shapes.append(layer_output_shape)
-
-        if mode in {'sum', 'mul', 'ave', 'cos', 'max'}:
-            input_shapes_set = set(input_shapes)
-            if len(input_shapes_set) > 1:
-                raise ValueError('Only layers of same output shape can '
-                                 'be merged using ' + mode + ' mode. ' +
-                                 'Layer shapes: %s' % input_shapes)
-        if mode in {'cos', 'dot'}:
-            if len(layers) > 2:
-                raise ValueError(mode + ' merge takes exactly 2 layers')
-            shape1 = input_shapes[0]
-            shape2 = input_shapes[1]
-            n1 = len(shape1)
-            n2 = len(shape2)
-            if isinstance(dot_axes, int):
-                if dot_axes < 0:
-                    self.dot_axes = [dot_axes % n1, dot_axes % n2]
-                else:
-                    self.dot_axes = [dot_axes, ] * 2
-            if not isinstance(self.dot_axes, (list, tuple)):
-                raise TypeError('Invalid type for dot_axes - '
-                                'should be a list.')
-            if len(self.dot_axes) != 2:
-                raise ValueError('Invalid format for dot_axes - '
-                                 'should contain two elements.')
-            if not isinstance(self.dot_axes[0], int) or not isinstance(self.dot_axes[1], int):
-                raise ValueError('Invalid format for dot_axes - '
-                                 'list elements should be "int".')
-            if shape1[self.dot_axes[0]] != shape2[self.dot_axes[1]]:
-                raise ValueError('Dimension incompatibility using dot mode: '
-                                 '%s != %s. ' % (shape1[self.dot_axes[0]], shape2[self.dot_axes[1]]) +
-                                 'Layer shapes: %s, %s' % (shape1, shape2))
-        elif mode == 'concat':
-            reduced_inputs_shapes = [list(shape) for shape in input_shapes]
-            shape_set = set()
-            for i in range(len(reduced_inputs_shapes)):
-                del reduced_inputs_shapes[i][self.concat_axis]
-                shape_set.add(tuple(reduced_inputs_shapes[i]))
-            if len(shape_set) > 1:
-                raise ValueError('"concat" mode can only merge '
-                                 'layers with matching '
-                                 'output shapes except for the concat axis. '
-                                 'Layer shapes: %s' % (input_shapes))
-
-    def call(self, inputs, mask=None):
-        if not isinstance(inputs, list) or len(inputs) <= 1:
-            raise TypeError('Merge must be called on a list of tensors '
-                            '(at least 2). Got: ' + str(inputs))
-        # Case: "mode" is a lambda or function.
-        if callable(self.mode):
-            arguments = self.arguments
-            if has_arg(self.mode, 'mask'):
-                arguments['mask'] = mask
-            return self.mode(inputs, **arguments)
-
-        if self.mode == 'sum' or self.mode == 'ave':
-            s = inputs[0]
-            for i in range(1, len(inputs)):
-                s += inputs[i]
-            if self.mode == 'ave':
-                s /= len(inputs)
-            return s
-
-        elif self.mode == 'concat':
-            return K.concatenate(inputs, axis=self.concat_axis)
-
-        elif self.mode == 'mul':
-            s = inputs[0]
-            for i in range(1, len(inputs)):
-                s *= inputs[i]
-            return s
-        elif self.mode == 'max':
-            s = inputs[0]
-            for i in range(1, len(inputs)):
-                s = K.maximum(s, inputs[i])
-            return s
-        elif self.mode == 'dot':
-            l1 = inputs[0]
-            l2 = inputs[1]
-            output = K.batch_dot(l1, l2, self.dot_axes)
-            return output
-
-        elif self.mode == 'cos':
-            l1 = inputs[0]
-            l2 = inputs[1]
-            denominator = K.sqrt(K.batch_dot(l1, l1, self.dot_axes) *
-                                 K.batch_dot(l2, l2, self.dot_axes))
-            denominator = K.maximum(denominator, K.epsilon())
-            output = K.batch_dot(l1, l2, self.dot_axes) / denominator
-            output = K.expand_dims(output, 1)
-            return output
-        else:
-            raise ValueError('Unknown merge mode.')
-
-    def compute_output_shape(self, input_shape):
-        # Must have multiple input shape tuples.
-        assert isinstance(input_shape, list)
-        # Case: callable self._output_shape.
-        if callable(self.mode):
-            if callable(self._output_shape):
-                output_shape = self._output_shape(input_shape)
-                return output_shape
-            elif self._output_shape is not None:
-                return (input_shape[0][0],) + tuple(self._output_shape)
-            else:
-                raise ValueError('The Merge layer ' + self.name +
-                                 ' has a callable `mode` argument, '
-                                 'and we cannot infer its output shape '
-                                 'because no `output_shape` '
-                                 'argument was provided. '
-                                 'Make sure to pass a shape tuple '
-                                 '(or callable) '
-                                 '`output_shape` to Merge.')
-        # Pre-defined merge modes.
-        input_shapes = input_shape
-        if self.mode in ['sum', 'mul', 'ave', 'max']:
-            # All tuples in input_shapes should be the same.
-            return input_shapes[0]
-        elif self.mode == 'concat':
-            output_shape = list(input_shapes[0])
-            for shape in input_shapes[1:]:
-                if output_shape[self.concat_axis] is None or shape[self.concat_axis] is None:
-                    output_shape[self.concat_axis] = None
-                    break
-                output_shape[self.concat_axis] += shape[self.concat_axis]
-            return tuple(output_shape)
-        elif self.mode in ['dot', 'cos']:
-            shape1 = list(input_shapes[0])
-            shape2 = list(input_shapes[1])
-            shape1.pop(self.dot_axes[0])
-            shape2.pop(self.dot_axes[1])
-            shape2.pop(0)
-            output_shape = shape1 + shape2
-            if len(output_shape) == 1:
-                output_shape += [1]
-            return tuple(output_shape)
-
-    def compute_mask(self, inputs, mask=None):
-        if mask is None or all([m is None for m in mask]):
-            return None
-
-        assert hasattr(mask, '__len__') and len(mask) == len(inputs)
-
-        if self.mode in ['sum', 'mul', 'ave', 'max']:
-            masks = [K.expand_dims(m, 0) for m in mask if m is not None]
-            return K.all(K.concatenate(masks, axis=0), axis=0, keepdims=False)
-        elif self.mode == 'concat':
-            # Make a list of masks while making sure
-            # the dimensionality of each mask
-            # is the same as the corresponding input.
-            masks = []
-            for input_i, mask_i in zip(inputs, mask):
-                if mask_i is None:
-                    # Input is unmasked. Append all 1s to masks,
-                    masks.append(K.ones_like(input_i, dtype='bool'))
-                elif K.ndim(mask_i) < K.ndim(input_i):
-                    # Mask is smaller than the input, expand it
-                    masks.append(K.expand_dims(mask_i))
-                else:
-                    masks.append(mask_i)
-            concatenated = K.concatenate(masks, axis=self.concat_axis)
-            return K.all(concatenated, axis=-1, keepdims=False)
-        elif self.mode in ['cos', 'dot']:
-            return None
-        elif callable(self.mode):
-            if callable(self._output_mask):
-                return self._output_mask(mask)
-            else:
-                return self._output_mask
-        else:
-            # This should have been caught earlier.
-            raise ValueError('Invalid merge mode: {}'.format(self.mode))
-
-    def get_config(self):
-        if isinstance(self.mode, python_types.LambdaType):
-            mode = func_dump(self.mode)
-            mode_type = 'lambda'
-        elif callable(self.mode):
-            mode = self.mode.__name__
-            mode_type = 'function'
-        else:
-            mode = self.mode
-            mode_type = 'raw'
-
-        if isinstance(self._output_shape, python_types.LambdaType):
-            output_shape = func_dump(self._output_shape)
-            output_shape_type = 'lambda'
-        elif callable(self._output_shape):
-            output_shape = self._output_shape.__name__
-            output_shape_type = 'function'
-        else:
-            output_shape = self._output_shape
-            output_shape_type = 'raw'
-
-        if isinstance(self._output_mask, python_types.LambdaType):
-            output_mask = func_dump(self._output_mask)
-            output_mask_type = 'lambda'
-        elif callable(self._output_mask):
-            output_mask = self._output_mask.__name__
-            output_mask_type = 'function'
-        else:
-            output_mask = self._output_mask
-            output_mask_type = 'raw'
-
-        return {'name': self.name,
-                'mode': mode,
-                'mode_type': mode_type,
-                'concat_axis': self.concat_axis,
-                'dot_axes': self.dot_axes,
-                'output_shape': output_shape,
-                'output_shape_type': output_shape_type,
-                'output_mask': output_mask,
-                'output_mask_type': output_mask_type,
-                'arguments': self.arguments}
-
-    @classmethod
-    def from_config(cls, config):
-        config = config.copy()
-        mode_type = config.pop('mode_type')
-        if mode_type == 'function':
-            mode = globals()[config['mode']]
-        elif mode_type == 'lambda':
-            mode = func_load(config['mode'], globs=globals())
-        else:
-            mode = config['mode']
-
-        output_shape_type = config.pop('output_shape_type', None)
-        if output_shape_type == 'function':
-            output_shape = globals()[config['output_shape']]
-        elif output_shape_type == 'lambda':
-            output_shape = func_load(config['output_shape'],
-                                     globs=globals())
-        else:
-            output_shape = config.get('output_shape')
-
-        output_mask_type = config.pop('output_mask_type', None)
-        if output_mask_type == 'function':
-            output_mask = globals()[config['output_mask']]
-        elif output_mask_type == 'lambda':
-            output_mask = func_load(config['output_mask'],
-                                    globs=globals())
-        else:
-            output_mask = config.get('output_mask')
-
-        config['mode'] = mode
-        config['output_shape'] = output_shape
-        config['output_mask'] = output_mask
-        return super(Merge, cls).from_config(config)
-
-
-def merge(inputs, mode='sum', concat_axis=-1,
-          dot_axes=-1, output_shape=None, output_mask=None,
-          arguments=None, name=None):
-    """Functional merge, to apply to Keras tensors (NOT layers).
-    Returns a Keras tensor.
-    # Example
-    ```python
-    tensor_a = Input(shape=(32,))
-    tensor_b = Input(shape=(32,))
-    merged_tensor = merge([tensor_a, tensor_b], mode='concat', concat_axis=1)
-    ```
-    # Arguments
-        mode: String or lambda/function. If string, must be one
-            of: 'sum', 'mul', 'concat', 'ave', 'cos', 'dot', 'max'.
-            If lambda/function, it should take as input a list of tensors
-            and return a single tensor.
-        concat_axis: Integer, axis to use in mode `concat`.
-        dot_axes: Integer or tuple of integers,
-            axes to use in mode `dot` or `cos`.
-        output_shape: Shape tuple (tuple of integers), or lambda/function
-            to compute output_shape (only if merge mode is a lambda/function).
-            If the latter case, it should take as input a list of shape tuples
-            (1:1 mapping to input tensors) and return a single shape tuple,
-            including the batch size
-            (same convention as the `compute_output_shape` method of layers).
-        node_indices: Optional list of integers containing
-            the output node index for each input layer
-            (in case some input layers have multiple output nodes).
-            will default to an array of 0s if not provided.
-        tensor_indices: Optional list of indices of output tensors
-            to consider for merging
-            (in case some input layer node returns multiple tensors).
-    """
-    warnings.warn('The `merge` function is deprecated '
-                  'and will be removed after 08/2017. '
-                  'Use instead layers from `keras.layers.merge`, '
-                  'e.g. `add`, `concatenate`, etc.', stacklevel=2)
-    all_keras_tensors = True
-    for x in inputs:
-        if not hasattr(x, '_keras_history'):
-            all_keras_tensors = False
-            break
-    if all_keras_tensors:
-        input_layers = []
-        node_indices = []
-        tensor_indices = []
-        for x in inputs:
-            input_layer, node_index, tensor_index = x._keras_history
-            input_layers.append(input_layer)
-            node_indices.append(node_index)
-            tensor_indices.append(tensor_index)
-        merge_layer = Merge(input_layers, mode=mode,
-                            concat_axis=concat_axis,
-                            dot_axes=dot_axes,
-                            output_shape=output_shape,
-                            output_mask=output_mask,
-                            arguments=arguments,
-                            node_indices=node_indices,
-                            tensor_indices=tensor_indices,
-                            name=name)
-        return merge_layer.inbound_nodes[0].output_tensors[0]
-    else:
-        merge_layer = Merge(mode=mode,
-                            concat_axis=concat_axis,
-                            dot_axes=dot_axes,
-                            output_shape=output_shape,
-                            output_mask=output_mask,
-                            arguments=arguments,
-                            name=name)
-        return merge_layer(inputs)
-
-
 class MaxoutDense(Layer):
     """A dense maxout layer.
     A `MaxoutDense` layer takes the element-wise maximum of
@@ -975,10 +522,8 @@ class Recurrent(Layer):
             # Compute the full input spec, including state
             input_spec = self.input_spec
             state_spec = self.state_spec
-            if not isinstance(input_spec, list):
-                input_spec = [input_spec]
-            if not isinstance(state_spec, list):
-                state_spec = [state_spec]
+            input_spec = to_list(input_spec)
+            state_spec = to_list(state_spec)
             self.input_spec = input_spec + state_spec
 
             # Compute the full inputs, including state
@@ -1114,3 +659,151 @@ class Recurrent(Layer):
                   'implementation': self.implementation}
         base_config = super(Recurrent, self).get_config()
         return dict(list(base_config.items()) + list(config.items()))
+
+
+class ConvRecurrent2D(Recurrent):
+    """Abstract base class for convolutional recurrent layers.
+
+    Do not use in a model -- it's not a functional layer!
+
+    # Arguments
+        filters: Integer, the dimensionality of the output space
+            (i.e. the number output of filters in the convolution).
+        kernel_size: An integer or tuple/list of n integers, specifying the
+            dimensions of the convolution window.
+        strides: An integer or tuple/list of n integers,
+            specifying the strides of the convolution.
+            Specifying any stride value != 1 is incompatible with specifying
+            any `dilation_rate` value != 1.
+        padding: One of `"valid"` or `"same"` (case-insensitive).
+        data_format: A string,
+            one of `channels_last` (default) or `channels_first`.
+            The ordering of the dimensions in the inputs.
+            `channels_last` corresponds to inputs with shape
+            `(batch, time, ..., channels)`
+            while `channels_first` corresponds to
+            inputs with shape `(batch, time, channels, ...)`.
+            It defaults to the `image_data_format` value found in your
+            Keras config file at `~/.keras/keras.json`.
+            If you never set it, then it will be "channels_last".
+        dilation_rate: An integer or tuple/list of n integers, specifying
+            the dilation rate to use for dilated convolution.
+            Currently, specifying any `dilation_rate` value != 1 is
+            incompatible with specifying any `strides` value != 1.
+        return_sequences: Boolean. Whether to return the last output
+            in the output sequence, or the full sequence.
+        go_backwards: Boolean (default False).
+            If True, process the input sequence backwards.
+        stateful: Boolean (default False). If True, the last state
+            for each sample at index i in a batch will be used as initial
+            state for the sample of index i in the following batch.
+
+    # Input shape
+        5D tensor with shape `(num_samples, timesteps, channels, rows, cols)`.
+
+    # Output shape
+        - if `return_sequences`: 5D tensor with shape
+            `(num_samples, timesteps, channels, rows, cols)`.
+        - else, 4D tensor with shape `(num_samples, channels, rows, cols)`.
+
+    # Masking
+        This layer supports masking for input data with a variable number
+        of timesteps. To introduce masks to your data,
+        use an [Embedding](embeddings.md) layer with the `mask_zero` parameter
+        set to `True`.
+        **Note:** for the time being, masking is only supported with Theano.
+
+    # Note on using statefulness in RNNs
+        You can set RNN layers to be 'stateful', which means that the states
+        computed for the samples in one batch will be reused as initial states
+        for the samples in the next batch.
+        This assumes a one-to-one mapping between
+        samples in different successive batches.
+
+        To enable statefulness:
+            - specify `stateful=True` in the layer constructor.
+            - specify a fixed batch size for your model, by passing
+                a `batch_input_size=(...)` to the first layer in your model.
+                This is the expected shape of your inputs *including the batch
+                size*.
+                It should be a tuple of integers, e.g. `(32, 10, 100)`.
+
+        To reset the states of your model, call `.reset_states()` on either
+        a specific layer, or on your entire model.
+    """
+
+    def __init__(self, filters,
+                 kernel_size,
+                 strides=(1, 1),
+                 padding='valid',
+                 data_format=None,
+                 dilation_rate=(1, 1),
+                 return_sequences=False,
+                 go_backwards=False,
+                 stateful=False,
+                 **kwargs):
+        super(ConvRecurrent2D, self).__init__(**kwargs)
+        self.filters = filters
+        self.kernel_size = conv_utils.normalize_tuple(kernel_size, 2, 'kernel_size')
+        self.strides = conv_utils.normalize_tuple(strides, 2, 'strides')
+        self.padding = conv_utils.normalize_padding(padding)
+        self.data_format = K.normalize_data_format(data_format)
+        self.dilation_rate = conv_utils.normalize_tuple(dilation_rate, 2, 'dilation_rate')
+        self.return_sequences = return_sequences
+        self.go_backwards = go_backwards
+        self.stateful = stateful
+        self.input_spec = [InputSpec(ndim=5)]
+        self.state_spec = None
+
+    def compute_output_shape(self, input_shape):
+        if isinstance(input_shape, list):
+            input_shape = input_shape[0]
+        if self.data_format == 'channels_first':
+            rows = input_shape[3]
+            cols = input_shape[4]
+        elif self.data_format == 'channels_last':
+            rows = input_shape[2]
+            cols = input_shape[3]
+        rows = conv_utils.conv_output_length(rows,
+                                             self.kernel_size[0],
+                                             padding=self.padding,
+                                             stride=self.strides[0],
+                                             dilation=self.dilation_rate[0])
+        cols = conv_utils.conv_output_length(cols,
+                                             self.kernel_size[1],
+                                             padding=self.padding,
+                                             stride=self.strides[1],
+                                             dilation=self.dilation_rate[1])
+        if self.return_sequences:
+            if self.data_format == 'channels_first':
+                output_shape = (input_shape[0], input_shape[1],
+                                self.filters, rows, cols)
+            elif self.data_format == 'channels_last':
+                output_shape = (input_shape[0], input_shape[1],
+                                rows, cols, self.filters)
+        else:
+            if self.data_format == 'channels_first':
+                output_shape = (input_shape[0], self.filters, rows, cols)
+            elif self.data_format == 'channels_last':
+                output_shape = (input_shape[0], rows, cols, self.filters)
+
+        if self.return_state:
+            if self.data_format == 'channels_first':
+                output_shape = [output_shape] + [(input_shape[0], self.filters, rows, cols) for _ in range(2)]
+            elif self.data_format == 'channels_last':
+                output_shape = [output_shape] + [(input_shape[0], rows, cols, self.filters) for _ in range(2)]
+
+        return output_shape
+
+    def get_config(self):
+        config = {'filters': self.filters,
+                  'kernel_size': self.kernel_size,
+                  'strides': self.strides,
+                  'padding': self.padding,
+                  'data_format': self.data_format,
+                  'dilation_rate': self.dilation_rate,
+                  'return_sequences': self.return_sequences,
+                  'go_backwards': self.go_backwards,
+                  'stateful': self.stateful}
+        base_config = super(ConvRecurrent2D, self).get_config()
+        return dict(list(base_config.items()) + list(config.items()))
