@@ -1,3 +1,9 @@
+"""Utilities used in convolutional layers.
+"""
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+
 from six.moves import range
 import numpy as np
 from .. import backend as K
@@ -42,17 +48,6 @@ def normalize_tuple(value, n, name):
     return value_tuple
 
 
-def normalize_data_format(value):
-    if value is None:
-        value = K.image_data_format()
-    data_format = value.lower()
-    if data_format not in {'channels_first', 'channels_last'}:
-        raise ValueError('The `data_format` argument must be one of '
-                         '"channels_first", "channels_last". Received: ' +
-                         str(value))
-    return data_format
-
-
 def normalize_padding(value):
     padding = value.lower()
     allowed = {'valid', 'same', 'causal'}
@@ -70,7 +65,7 @@ def convert_kernel(kernel):
     Also works reciprocally, since the transformation is its own inverse.
 
     # Arguments
-        kernel: Numpy array (4D or 5D).
+        kernel: Numpy array (3D, 4D or 5D).
 
     # Returns
         The converted kernel.
@@ -79,7 +74,7 @@ def convert_kernel(kernel):
         ValueError: in case of invalid kernel shape or invalid data_format.
     """
     kernel = np.asarray(kernel)
-    if not 4 <= kernel.ndim <= 5:
+    if not 3 <= kernel.ndim <= 5:
         raise ValueError('Invalid kernel shape:', kernel.shape)
     slices = [slice(None, None, -1) for _ in range(kernel.ndim)]
     no_flip = (slice(None, None), slice(None, None))
@@ -140,13 +135,42 @@ def conv_input_length(output_length, filter_size, padding, stride):
     return (output_length - 1) * stride - 2 * pad + filter_size
 
 
-def deconv_length(dim_size, stride_size, kernel_size, padding):
+def deconv_length(dim_size, stride_size, kernel_size, padding, output_padding):
+    """Determines output length of a transposed convolution given input length.
+
+    # Arguments
+        dim_size: Integer, the input length.
+        stride_size: Integer, the stride along the dimension of `dim_size`.
+        kernel_size: Integer, the kernel size along the dimension of
+            `dim_size`.
+        padding: One of `"same"`, `"valid"`, `"full"`.
+        output_padding: Integer, amount of padding along the output dimension,
+            Can be set to `None` in which case the output length is inferred.
+
+    # Returns
+        The output length (integer).
+    """
+    assert padding in {'same', 'valid', 'full'}
     if dim_size is None:
         return None
-    if padding == 'valid':
-        dim_size = dim_size * stride_size + max(kernel_size - stride_size, 0)
-    elif padding == 'full':
-        dim_size = dim_size * stride_size - (stride_size + kernel_size - 2)
-    elif padding == 'same':
-        dim_size = dim_size * stride_size
+
+    # Infer length if output padding is None, else compute the exact length
+    if output_padding is None:
+        if padding == 'valid':
+            dim_size = dim_size * stride_size + max(kernel_size - stride_size, 0)
+        elif padding == 'full':
+            dim_size = dim_size * stride_size - (stride_size + kernel_size - 2)
+        elif padding == 'same':
+            dim_size = dim_size * stride_size
+    else:
+        if padding == 'same':
+            pad = kernel_size // 2
+        elif padding == 'valid':
+            pad = 0
+        elif padding == 'full':
+            pad = kernel_size - 1
+
+        dim_size = ((dim_size - 1) * stride_size + kernel_size - 2 * pad +
+                    output_padding)
+
     return dim_size
